#ifndef _LIB_DEBUG_H_
#define _LIB_DEBUG_H_

#include <errno.h>


#define DEBUG_BLOCKSIZE      8192
#define DEBUG_INDENT_WIDTH   3


#ifndef USE_LIBMEM_FUNC
#  define MEMalloc(a)       malloc (a)
#  define MEMrealloc(a,b)   realloc ((a), (b))
#endif

#ifdef DEBUG
#  define debug_noreturn      { debug_print ("}"); return; }
#  define debug_return(a,b)   { debug_print ((a), (b)); return (b); }
#  define debug(a)            { __errno__ = errno; debug_print a; errno = __errno__; }
#  define debug_endfunc       debug_print ("}");
#  define debug_main_arg      {                                                                                                            \
                                 short _i_;                                                                                                \
                                                                                                                                           \
                                 debug_print ("{%s(argc = %d, argv = 0x%08x, envp = 0x%08x)", __FUNCTION__, argc, (int) argv, (int) envp); \
                                                                                                                                           \
                                 debug_print ("argv = {\n");                                                                               \
                                 for (_i_ = 0; _i_ < argc; _i_++)                                                                          \
                                    debug_print ("   %2d> \"%s\"\n", _i_, argv[_i_]);                                                      \
                                 debug_print (" }\n");                                                                                     \
                                                                                                                                           \
                                 if (envp)                                                                                                 \
                                 {                                                                                                         \
                                    debug_print ("envp = {\n");                                                                            \
                                    for (_i_ = 0; envp[_i_]; _i_++)                                                                        \
                                       debug_print ("   %2d> \"%s\"\n", _i_, envp[_i_]);                                                   \
                                    debug_print (" }\n");                                                                                  \
                                 }                                                                                                         \
                              }


extern int __errno__;
#else

#ifdef ANSI_FUNC

static inline void debug_dummy (void)
#else

static inline void debug_dummy ()
#endif
{}

#  define debug_noreturn      return;
#  define debug_return(a,b)   return (b);
#  define debug(a)            ;
#  define debug_endfunc       ;
#  define debug_main_arg      ;
#  define debug_open(a)
#  define debug_close         debug_dummy
#  define debug_on            debug_dummy
#  define debug_off           debug_dummy
#  define debug_flush         debug_dummy
#  define debug_addtext(a)
#endif /* DEBUG */


#if defined(__STDC__) || defined(__cplusplus) || defined(NeedFunctionPrototypes)
#  define __PARMS(s) s
#else
#  define __PARMS(s) ()
#endif

void   perror_exit   __PARMS((char *msg, short errnum));
time_t timeval_diff  __PARMS((struct timeval *tv_1, struct timeval *tv_2));

#ifdef DEBUG
void   debug_close   __PARMS((void));
void   debug_on      __PARMS((void));
void   debug_off     __PARMS((void));
void   debug_flush   __PARMS((void));
void   debug_open    __PARMS((char *filename));
void   debug_addtext __PARMS((char *text));
void   debug_print   __PARMS((char *format, ...));
#endif

#undef __PARMS
#endif /* _LIB_DEBUG_H_ */
